import json
import os
from typing import List


def get_config(config_dir: str):
    config_path = os.path.join(config_dir, ".license-sh.json")
    try:
        with open(config_path) as file:
            return json.load(file)
    except FileNotFoundError:
        return {}


def write_config(config_dir: str, config):
    config_path = os.path.join(config_dir, ".license-sh.json")
    try:
        with open(config_path, "w+") as outfile:
            json.dump(config, outfile, indent=2, sort_keys=True)
            return True
    except FileNotFoundError:
        return False


def whitelist_licenses(config_dir: str, licenses: List[str]):
    config = get_config(config_dir)
    config["whitelist"] = list(set(config.get("whitelist", []) + licenses))
    write_config(config_dir, config)



import unittest
import json

from license_sh.runners.yarn import (
    parse_yarn_lock,
    get_name,
    get_flat_tree,
    get_dependency_tree,
)


class ConsoleReporterTestCase(unittest.TestCase):
    def test_get_name_simple(self):
        name = get_name("test@1.2.3")
        self.assertEqual(name, "test")

    def test_get_name_simple_caret(self):
        name = get_name("test@^1.2.3")
        self.assertEqual(name, "test")

    def test_get_sign_name(self):
        name = get_name("@test@^1.2.3")
        self.assertEqual(name, "@test")

    def test_empty_lock(self):
        lock_file = """{
  "type": "success",
  "object": {}
}
"""
        package_map = parse_yarn_lock(json.loads(lock_file))
        self.assertFalse(package_map)

    def test_single_child_lock(self):
        lock_file = """{
  "type": "success",
  "object": {
    "toolbox-core@^1.0.1": {
      "version": "1.1.1"
    }
  }
}
"""
        package_map = parse_yarn_lock(json.loads(lock_file))
        self.assertEqual(package_map.get("toolbox-core@^1.0.1"), "1.1.1")

    def test_sign_child_lock(self):
        lock_file = """{
  "type": "success",
  "object": {
    "@toolbox-core@^1.0.1": {
      "version": "1.1.1"
    }
  }
}
"""
        package_map = parse_yarn_lock(json.loads(lock_file))
        self.assertEqual(package_map.get("@toolbox-core@^1.0.1"), "1.1.1")

    def test_multiple_children_lock(self):
        lock_file = """{
  "type": "success",
  "object": {
    "toolbox-core@^1.0.1": {
      "version": "1.1.1"
    },
    "toolbox-core1@^1.0.1": {
      "version": "1.1.2"
    },
    "toolbox-core2@^1.0.1": {
      "version": "1.1.3"
    }
  }
}
"""
        package_map = parse_yarn_lock(json.loads(lock_file))
        self.assertEqual(package_map.get("toolbox-core@^1.0.1"), "1.1.1")
        self.assertEqual(package_map.get("toolbox-core1@^1.0.1"), "1.1.2")
        self.assertEqual(package_map.get("toolbox-core2@^1.0.1"), "1.1.3")

    def test_single_child_flat_tree(self):
        tree_data = [{"name": "@babel/helper-create-class-features-plugin@^7.5.5"}]
        package_map = {"@babel/helper-create-class-features-plugin@^7.5.5": "1.1.3"}
        tree = get_flat_tree(tree_data, package_map)
        elem = tree.get("@babel/helper-create-class-features-plugin@^7.5.5")
        self.assertEqual(elem.get("name"), "@babel/helper-create-class-features-plugin")
        self.assertEqual(elem.get("version"), "1.1.3")

    def test_nested_children_flat_tree(self):
        tree_data = [
            {
                "name": "@babel/helper-create-class-features-plugin@^7.5.5",
                "children": [
                    {"name": "@babel/helper-plugin-utils@^7.0.0"},
                    {"name": "@babel/plugin-syntax-object-rest-spread@^7.2.0"},
                ],
            }
        ]
        package_map = {
            "@babel/helper-create-class-features-plugin@^7.5.5": "1.1.3",
            "@babel/helper-plugin-utils@^7.0.0": "9.9.9",
            "@babel/plugin-syntax-object-rest-spread@^7.2.0": "1.5.6",
        }
        tree = get_flat_tree(tree_data, package_map)
        elem = (
            tree.get("@babel/helper-create-class-features-plugin@^7.5.5")
            .get("dependencies")
            .get("@babel/plugin-syntax-object-rest-spread@^7.2.0")
        )
        self.assertEqual(elem.get("name"), "@babel/plugin-syntax-object-rest-spread")
        self.assertEqual(elem.get("version"), "1.5.6")

    def test_children_with_nested_child_flat_tree(self):
        tree_data = [
            {
                "name": "@babel/helper-create-class-features-plugin@^7.5.5",
                "children": [
                    {"name": "@babel/helper-plugin-utils@^7.0.0"},
                    {"name": "@babel/plugin-syntax-object-rest-spread@^7.2.0"},
                ],
            },
            {
                "name": "class-features-plugin@^7.5.5",
                "children": [{"name": "utils@^7.0.0"}],
            },
        ]
        package_map = {
            "@babel/helper-create-class-features-plugin@^7.5.5": "1.1.3",
            "@babel/helper-plugin-utils@^7.0.0": "9.9.9",
            "@babel/plugin-syntax-object-rest-spread@^7.2.0": "1.5.6",
            "class-features-plugin@^7.5.5": "8.5.3",
            "utils@^7.0.0": "6.5.2",
        }
        tree = get_flat_tree(tree_data, package_map)
        elem = (
            tree.get("@babel/helper-create-class-features-plugin@^7.5.5")
            .get("dependencies")
            .get("@babel/plugin-syntax-object-rest-spread@^7.2.0")
        )
        self.assertEqual(elem.get("name"), "@babel/plugin-syntax-object-rest-spread")
        self.assertEqual(elem.get("version"), "1.5.6")

        elem2 = (
            tree.get("class-features-plugin@^7.5.5")
            .get("dependencies")
            .get("utils@^7.0.0")
        )
        self.assertEqual(elem2.get("name"), "utils")
        self.assertEqual(elem2.get("version"), "6.5.2")

    def test_empty_dep_tree(self):
        package_json = {"name": "license", "version": "7.8.8", "dependencies": {}}
        tree_data = {}

        tree = get_dependency_tree(tree_data, package_json, {})
        self.assertEqual(tree.name, "license")
        self.assertEqual(tree.version, "7.8.8")

    def test_simple_dep_tree(self):
        package_json = {
            "name": "license",
            "version": "7.8.8",
            "dependencies": {"@babel/helper-create-class-features-plugin": "7.5.5"},
        }
        tree_data = {
            "@babel/helper-create-class-features-plugin@7.5.5": {
                "name": "@babel/helper-create-class-features-plugin",
                "version": "7.5.5",
                "dependencies": {
                    "@babel/helper-plugin-utils@7.0.0": {
                        "name": "@babel/helper-plugin-utils",
                        "version": "7.0.0",
                        "dependencies": {
                            "plugin-utils@7.0.0": {
                                "name": "plugin-utils",
                                "version": "7.0.0",
                            }
                        },
                    }
                },
            }
        }

        package_map = {
            "@babel/helper-create-class-features-plugin@7.5.5": "7.5.5",
            "@babel/helper-plugin-utils@7.0.0": "7.0.0",
            "plugin-utils@7.0.0": "7.0.0",
        }
        tree = get_dependency_tree(tree_data, package_json, package_map)
        self.assertEqual(
            tree.children[0].name, "@babel/helper-create-class-features-plugin"
        )
        self.assertEqual(tree.children[0].version, "7.5.5")
        self.assertEqual(
            tree.children[0].children[0].name, "@babel/helper-plugin-utils"
        )
        self.assertEqual(tree.children[0].children[0].version, "7.0.0")
        self.assertEqual(tree.children[0].children[0].children[0].name, "plugin-utils")
        self.assertEqual(tree.children[0].children[0].children[0].version, "7.0.0")

    def test_multiple_main_dep_tree(self):
        package_json = {
            "name": "license",
            "version": "7.8.8",
            "dependencies": {
                "@babel/helper-create-class-features-plugin": "7.5.5",
                "plugin": "7.5.0",
                "@license/plugin": "0.5.5",
            },
        }
        tree_data = {
            "@babel/helper-create-class-features-plugin@7.5.5": {
                "name": "@babel/helper-create-class-features-plugin",
                "version": "1.2.3",
                "dependencies": {},
            },
            "plugin@7.5.0": {"name": "plugin", "version": "1.2.3", "dependencies": {}},
            "@license/plugin@0.5.5": {
                "name": "plugin",
                "version": "0.5.5",
                "dependencies": {},
            },
        }

        package_map = {
            "@babel/helper-create-class-features-plugin@7.5.5": "7.5.5",
            "@license/plugin@0.5.5": "0.5.5",
            "plugin@7.5.0": "7.5.0",
        }
        tree = get_dependency_tree(tree_data, package_json, package_map)
        self.assertEqual(len(tree.children), 3)
        self.assertEqual(
            tree.children[0].name, "@babel/helper-create-class-features-plugin"
        )
        self.assertEqual(tree.children[0].version, "1.2.3")
        self.assertEqual(tree.children[1].name, "plugin")
        self.assertEqual(tree.children[1].version, "1.2.3")
        self.assertEqual(tree.children[2].name, "@license/plugin")
        self.assertEqual(tree.children[2].version, "0.5.5")

    def test_shared_dep_tree(self):
        package_json = {
            "name": "license",
            "version": "7.8.8",
            "dependencies": {
                "@babel/helper-create-class-features-plugin": "7.5.5",
                "plugin": "7.5.0",
            },
        }
        tree_data = {
            "@babel/helper-create-class-features-plugin@7.5.5": {
                "name": "@babel/helper-create-class-features-plugin",
                "version": "1.2.3",
                "dependencies": {
                    "@license/plugin@0.5.5": {
                        "name": "@license/plugin",
                        "version": "0.5.5",
                        "dependencies": {
                            "shared@0.5.5": {
                                "name": "shared",
                                "version": "0.5.5",
                                "dependencies": {},
                            }
                        },
                    }
                },
            },
            "plugin@7.5.0": {
                "name": "plugin",
                "version": "1.2.3",
                "dependencies": {
                    "shared@0.5.5": {
                        "name": "shared",
                        "version": "0.5.5",
                        "dependencies": {},
                    }
                },
            },
            "shared@0.5.5": {
                "name": "shared",
                "version": "0.5.5",
                "dependencies": {
                    "shared_child@0.5.5": {
                        "name": "shared_child",
                        "version": "0.5.5",
                        "dependencies": {},
                    }
                },
            },
        }
        package_map = {
            "@babel/helper-create-class-features-plugin@7.5.5": "7.5.5",
            "@license/plugin@0.5.5": "0.5.5",
            "shared_child@0.5.5": "0.5.5",
            "plugin@7.5.0": "7.5.0",
            "shared@0.5.5": "0.5.5",
        }
        tree = get_dependency_tree(tree_data, package_json, package_map)
        self.assertEqual(len(tree.children), 2)
        self.assertEqual(
            tree.children[0].name, "@babel/helper-create-class-features-plugin"
        )
        self.assertEqual(tree.children[0].version, "1.2.3")
        self.assertEqual(tree.children[0].children[0].name, "@license/plugin")
        self.assertEqual(tree.children[0].children[0].version, "0.5.5")
        self.assertEqual(tree.children[0].children[0].children[0].name, "shared")
        self.assertEqual(tree.children[0].children[0].children[0].version, "0.5.5")
        self.assertEqual(
            tree.children[0].children[0].children[0].children[0].name, "shared_child"
        )
        self.assertEqual(
            tree.children[0].children[0].children[0].children[0].version, "0.5.5"
        )
        self.assertEqual(tree.children[1].name, "plugin")
        self.assertEqual(tree.children[1].version, "1.2.3")
        self.assertEqual(tree.children[1].children[0].name, "shared")
        self.assertEqual(tree.children[1].children[0].version, "0.5.5")
        self.assertEqual(tree.children[1].children[0].children[0].name, "shared_child")
        self.assertEqual(tree.children[1].children[0].children[0].version, "0.5.5")

    def test_simple_optional_dep_tree(self):
        package_json = {
            "name": "license",
            "version": "7.8.8",
            "dependencies": {"@babel/helper-create-class-features-plugin": "^7.5.5"},
        }
        tree_data = {
            "@babel/helper-create-class-features-plugin@7.5.5": {
                "name": "@babel/helper-create-class-features-plugin",
                "version": "7.5.5",
            }
        }
        package_map = {"@babel/helper-create-class-features-plugin@^7.5.5": "7.5.5"}
        tree = get_dependency_tree(tree_data, package_json, package_map)
        self.assertEqual(len(tree.children), 1)
        self.assertEqual(
            tree.children[0].name, "@babel/helper-create-class-features-plugin"
        )
        self.assertEqual(tree.children[0].version, "7.5.5")

    def test_circular_dep_tree(self):
        package_json = {
            "name": "license",
            "version": "7.8.8",
            "dependencies": {"@babel/helper-create-class-features-plugin": "7.5.5"},
        }
        tree_data = {
            "@babel/helper-create-class-features-plugin@7.5.5": {
                "name": "@babel/helper-create-class-features-plugin",
                "version": "7.5.5",
                "dependencies": {
                    "features-plugin@7.5.5": {
                        "name": "features-plugin",
                        "version": "7.5.5",
                    }
                },
            },
            "features-plugin@7.5.5": {
                "name": "features-plugin",
                "version": "7.5.5",
                "dependencies": {
                    "@babel/helper-create-class-features-plugin@7.5.5": {
                        "name": "@babel/helper-create-class-features-plugin",
                        "version": "7.5.5",
                    }
                },
            },
        }
        package_map = {"@babel/helper-create-class-features-plugin@7.5.5": "7.5.5"}
        tree = get_dependency_tree(tree_data, package_json, package_map)
        self.assertEqual(
            tree.children[0].name, "@babel/helper-create-class-features-plugin"
        )
        self.assertEqual(tree.children[0].version, "7.5.5")
        self.assertEqual(tree.children[0].children[0].name, "features-plugin")
        self.assertEqual(tree.children[0].children[0].version, "7.5.5")
        self.assertEqual(
            tree.children[0].children[0].children[0].name,
            "@babel/helper-create-class-features-plugin",
        )
        self.assertEqual(tree.children[0].children[0].children[0].version, "7.5.5")
        self.assertEqual(len(tree.children[0].children[0].children[0].children), 0)

    def test_same_nested_dep_tree(self):
        package_json = {
            "name": "license",
            "version": "7.8.8",
            "dependencies": {"@babel/helper-create-class-features-plugin": "7.5.5"},
        }
        tree_data = {
            "@babel/helper-create-class-features-plugin@7.5.5": {
                "name": "@babel/helper-create-class-features-plugin",
                "version": "7.5.5",
                "dependencies": {
                    "features-plugin@^7.5.5": {
                        "name": "features-plugin",
                        "version": "7.5.5",
                    },
                    "features-plugin@7.5.5": {
                        "name": "features-plugin",
                        "version": None,
                        "dependencies": {
                            "@babel/plugin@7.5.5": {
                                "name": "@babel/plugin",
                                "version": "7.5.5",
                            }
                        },
                    },
                },
            }
        }
        package_map = {
            "@babel/helper-create-class-features-plugin@7.5.5": "7.5.5",
            "features-plugin@^7.5.5": "7.5.5",
        }
        tree = get_dependency_tree(tree_data, package_json, package_map)
        self.assertEqual(
            tree.children[0].name, "@babel/helper-create-class-features-plugin"
        )
        self.assertEqual(tree.children[0].version, "7.5.5")
        self.assertEqual(len(tree.children[0].children), 1)
        self.assertEqual(tree.children[0].children[0].name, "features-plugin")
        self.assertEqual(tree.children[0].children[0].version, "7.5.5")
        self.assertEqual(tree.children[0].children[0].children[0].name, "@babel/plugin")
        self.assertEqual(tree.children[0].children[0].children[0].version, "7.5.5")


if __name__ == "__main__":
    unittest.main()



import os
from os import access, R_OK
from os.path import isfile
from enum import Enum


class ProjectType(Enum):
    PYTHON_PIPENV = "python_pipenv"
    NPM = "npm"
    MAVEN = "maven"
    YARN = "yarn"


__file_exists = lambda path: isfile(path) and access(path, R_OK)


def get_project_types(dir_path: str) -> [ProjectType]:
    types = []

    # check for python's pipenv
    pipfile_path: str = os.path.join(dir_path, "Pipfile")
    pipfile_lock_path: str = os.path.join(dir_path, "Pipfile.lock")

    if __file_exists(pipfile_path) and __file_exists(pipfile_lock_path):
        types += [ProjectType.PYTHON_PIPENV]

    # check for npm
    pipfile_path: str = os.path.join(dir_path, "package.json")
    pipfile_lock_path: str = os.path.join(dir_path, "package-lock.json")

    if __file_exists(pipfile_path) and __file_exists(pipfile_lock_path):
        types += [ProjectType.NPM]

    # check for maven
    maven_path: str = os.path.join(dir_path, "pom.xml")

    if __file_exists(maven_path):
        types += [ProjectType.MAVEN]
    # check for yarn
    yarn_path: str = os.path.join(dir_path, "package.json")
    yarn_lock_path: str = os.path.join(dir_path, "yarn.lock")
    if __file_exists(yarn_path) and __file_exists(yarn_lock_path):
        types += [ProjectType.YARN]

    return types



import json
import asyncio
import aiohttp as aiohttp

NPM_HOST = "https://registry.npmjs.org"


def fetch_npm_licenses(all_dependencies):
    license_map = {}

    urls = [
        (f"{NPM_HOST}/{dependency}/", version)
        for dependency, version in all_dependencies
    ]

    async def fetch(session, url, version):
        async with session.get(url) as resp:
            return await resp.text(), version
            # Catch HTTP errors/exceptions here

    async def fetch_concurrent(urls):
        loop = asyncio.get_event_loop()
        async with aiohttp.ClientSession() as session:
            tasks = []
            for url, version in urls:
                tasks.append(loop.create_task(fetch(session, url, version)))

            for result in asyncio.as_completed(tasks):
                try:
                    output, version = await result
                    page = json.loads(output)
                    license_map[f"{page['name']}@{version}"] = page.get(
                        "license", "Unknown"
                    )
                except json.JSONDecodeError:
                    pass

    asyncio.run(fetch_concurrent(urls))

    return license_map



import unittest
from license_sh.runners.maven import (
    parse_dependency_xml,
    parse_licenses_xml,
    get_project_name,
)
import xml.etree.ElementTree as ET


class ParserTestCase(unittest.TestCase):
    def test_project_name(self):
        pom_xml = """<?xml version="1.0" encoding="UTF-8"?>
<project
  xmlns="http://maven.apache.org/POM/4.0.0"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
  http://maven.apache.org/xsd/maven-4.0.0.xsd"
>
  <modelVersion>4.0.0</modelVersion>
  <groupId>link.sharpe</groupId>
  <artifactId>mavenproject1</artifactId>
  <version>1.0-SNAPSHOT</version>
</project>
"""
        name = get_project_name(ET.fromstring(pom_xml))
        self.assertEqual(name, "mavenproject1")

    def test_none_tree(self):
        tree = parse_dependency_xml(None)
        self.assertIsNone(tree)

    def test_empty_tree(self):
        tree_text = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<license-tree version="0.0.1-SNAPSHOT">
</license-tree>
    """
        tree = parse_dependency_xml(ET.fromstring(tree_text))
        self.assertTrue(tree)
        self.assertEqual(tree.name, "license-tree")
        self.assertEqual(tree.version, "0.0.1-SNAPSHOT")
        self.assertEqual(len(tree.children), 0)

    def test_single_child_tree(self):
        tree_text = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<license-tree version="0.0.1-SNAPSHOT">
<spring-boot-starter-data-jpa version="2.1.8.RELEASE" />
</license-tree>
    """
        tree = parse_dependency_xml(ET.fromstring(tree_text))
        self.assertEqual(len(tree.children), 1)
        self.assertEqual(tree.children[0].name, "spring-boot-starter-data-jpa")
        self.assertEqual(tree.children[0].version, "2.1.8.RELEASE")

    def test_two_children_tree(self):
        tree_text = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<license-tree version="0.0.1-SNAPSHOT">
  <spring-boot-starter-data-jpa version="2.1.8.RELEASE" />
  <atmosphere-runtime version="2.4.30.slf4jvaadin1"/>
</license-tree>
    """
        tree = parse_dependency_xml(ET.fromstring(tree_text))
        self.assertEqual(len(tree.children), 2)
        self.assertEqual(tree.children[1].name, "atmosphere-runtime")
        self.assertEqual(tree.children[1].version, "2.4.30.slf4jvaadin1")

    def test_nested_child_tree(self):
        tree_text = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<license-tree version="0.0.1-SNAPSHOT">
  <spring-boot-starter-data-jpa version="2.1.8.RELEASE">
    <atmosphere-runtime version="2.4.30.slf4jvaadin1"/>
  </spring-boot-starter-data-jpa>
</license-tree>
    """
        tree = parse_dependency_xml(ET.fromstring(tree_text))
        self.assertEqual(len(tree.children), 1)
        self.assertEqual(tree.children[0].children[0].name, "atmosphere-runtime")
        self.assertEqual(tree.children[0].children[0].version, "2.4.30.slf4jvaadin1")

    def test_nested_children_tree(self):
        tree_text = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<license-tree version="0.0.1-SNAPSHOT">
  <spring-boot-starter-data-jpa version="2.1.8.RELEASE">
    <atmosphere-runtime version="2.4.30.slf4jvaadin1"/>
    <jackson-datatype-jdk8 version="2.9.9"/>
    <vaadin-context-menu-flow version="3.0.2">
      <vaadin-context-menu version="4.3.12"/>
    </vaadin-context-menu-flow>
  </spring-boot-starter-data-jpa>
  <spring-core version="5.1.9.RELEASE">
    <spring-jcl version="5.1.9.RELEASE"/>
  </spring-core>
</license-tree>
    """
        tree = parse_dependency_xml(ET.fromstring(tree_text))
        self.assertEqual(tree.name, "license-tree")
        self.assertEqual(tree.version, "0.0.1-SNAPSHOT")
        self.assertEqual(tree.children[0].name, "spring-boot-starter-data-jpa")
        self.assertEqual(tree.children[0].version, "2.1.8.RELEASE")
        self.assertEqual(tree.children[0].children[0].name, "atmosphere-runtime")
        self.assertEqual(tree.children[0].children[0].version, "2.4.30.slf4jvaadin1")
        self.assertEqual(tree.children[0].children[1].name, "jackson-datatype-jdk8")
        self.assertEqual(tree.children[0].children[1].version, "2.9.9")
        self.assertEqual(tree.children[0].children[2].name, "vaadin-context-menu-flow")
        self.assertEqual(tree.children[0].children[2].version, "3.0.2")
        self.assertEqual(
            tree.children[0].children[2].children[0].name, "vaadin-context-menu"
        )
        self.assertEqual(tree.children[0].children[2].children[0].version, "4.3.12")
        self.assertEqual(tree.children[1].name, "spring-core")
        self.assertEqual(tree.children[1].version, "5.1.9.RELEASE")
        self.assertEqual(tree.children[1].children[0].name, "spring-jcl")
        self.assertEqual(tree.children[1].children[0].version, "5.1.9.RELEASE")

    def test_parse_licenses_xml(self):
        license_text = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<licenseSummary>
  <dependencies>
    <dependency>
      <groupId>antlr</groupId>
      <artifactId>antlr</artifactId>
      <version>2.7.7</version>
      <licenses>
        <license>
          <name>BSD License</name>
          <url>http://www.antlr.org/license.html</url>
          <distribution>repo</distribution>
          <file>bsd license - license.html</file>
        </license>
      </licenses>
    </dependency>
    <dependency>
      <groupId>ch.qos.logback</groupId>
      <artifactId>logback-classic</artifactId>
      <version>1.2.3</version>
      <licenses>
        <license>
          <name>Eclipse Public License - v 1.0</name>
          <url>http://www.eclipse.org/legal/epl-v10.html</url>
          <file>eclipse public license - v 1.0 - epl-v10.html</file>
        </license>
        <license>
          <name>GNU Lesser General Public License</name>
          <url>http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html</url>
          <file>gnu lesser general public license - lgpl-2.1.html</file>
        </license>
      </licenses>
    </dependency>
    <dependency>
      <groupId>ch.qos.logback</groupId>
      <artifactId>logback-core</artifactId>
      <version>1.2.3</version>
      <licenses>
        <license>
          <name>Eclipse Public License - v 1.0</name>
          <url>http://www.eclipse.org/legal/epl-v10.html</url>
          <file>eclipse public license - v 1.0 - epl-v10.html</file>
        </license>
        <license>
          <name>GNU Lesser General Public License</name>
          <url>http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html</url>
          <file>gnu lesser general public license - lgpl-2.1.html</file>
        </license>
      </licenses>
    </dependency>
  </dependencies>
</licenseSummary>
"""

        license_map = parse_licenses_xml(ET.fromstring(license_text))
        self.assertEqual(license_map["antlr@2.7.7"], "BSD License")
        self.assertEqual(
            license_map["logback-classic@1.2.3"],
            "(Eclipse Public License - v 1.0 AND GNU Lesser General Public License)",
        )
        self.assertEqual(
            license_map["logback-core@1.2.3"],
            "(Eclipse Public License - v 1.0 AND GNU Lesser General Public License)",
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from license_sh.helpers import is_license_ok


class LicenseValidationsTestCase(unittest.TestCase):
    def test_simple_validation_test(self):
        whitelist = ["MIT", "Apache-2.0"]
        self.assertTrue(
            is_license_ok("MIT", whitelist),
            "MIT should not be not problematic, it's in a whitelist",
        )

    def test_license_in_comma_separated_expression(self):
        whitelist = ["MIT", "Apache-2.0"]
        # license.sh doesn't support commas, we support SPDX syntax
        self.assertEqual(is_license_ok("MIT, Apache 2.0", whitelist), False)

    def test_unparsable_expression(self):
        whitelist = ["MIT", "The Apache Software License, Version 2.0"]
        self.assertEqual(
            is_license_ok("The Apache Software License, Version 2.0", whitelist), True
        )

    def test_and_license_expression(self):
        whitelist = ["MIT", "Apache-2.0"]
        self.assertTrue(is_license_ok("(MIT AND Apache-2.0)", whitelist))
        self.assertFalse(is_license_ok("(MIT AND Apache-3.0)", whitelist))

    def test_or_license_expression(self):
        whitelist = ["MIT", "Apache-3.0"]
        self.assertTrue(is_license_ok("(MIT OR Apache-2.0)", whitelist))

    def test_complex_or_license_expression(self):
        whitelist = ["MIT", "Apache-3.0"]
        self.assertTrue(
            is_license_ok("((MIT and Apache-3.0) OR Apache-2.0)", whitelist)
        )
        self.assertTrue(
            is_license_ok("((License-2 or Apache-3.0) OR Apache-2.0)", whitelist)
        )
        self.assertFalse(
            is_license_ok("((License-2 and Apache-3.0) OR Apache-2.0)", whitelist)
        )

    def test_nested_invalid_expression_inside_complex_or_license_expression(self):
        whitelist = ["MIT", "Apache-3.0"]
        self.assertFalse(
            is_license_ok("((License-2, Apache-3.0) OR Apache-2.0)", whitelist)
        )

    def test_multi_operator_expression(self):
        whitelist = ["MIT"]
        self.assertEqual(
            is_license_ok("(BSD-2-Clause OR MIT OR Apache-2.0)", whitelist), True
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from anytree import AnyNode
from anytree.exporter import DictExporter
from license_sh.helpers import flatten_dependency_tree, annotate_dep_tree


def get_tree():
    """
  Please always re-generate this comment when changing the tree.
  You can use `print(RenderTree(tree))`

  :return:
  AnyNode(license='MIT', license_problem=False, name='Name', subtree_problem=True, version='')
  |-- AnyNode(license='MIT', license_problem=False, name='@company/package1', subtree_problem=True, version='1.1.1')
  |   |-- AnyNode(license='MIT', license_problem=False, name='package2', subtree_problem=False, version='2.2.2')
  |   |   |-- AnyNode(license='MIT', license_problem=False, name='package5', subtree_problem=False, version='5.5.5')
  |   |   +-- AnyNode(license='MIT', license_problem=False, name='package7', subtree_problem=False, version='7.7.7')
  |   |-- AnyNode(license='MIT', license_problem=False, name='package3', subtree_problem=True, version='3.3.3')
  |   |   +-- AnyNode(license='GPL', license_problem=True, name='package7', subtree_problem=False, version='7.7.6')
  |   |-- AnyNode(license='MIT', license_problem=False, name='package4', subtree_problem=False, version='4.4.4')
  |   +-- AnyNode(license='MIT', license_problem=False, name='package5', subtree_problem=True, version='5.5.5')
  |       +-- AnyNode(license='GPL', license_problem=True, name='package6', subtree_problem=False, version='6.6.6')
  +-- AnyNode(license='MIT', license_problem=False, name='package4', subtree_problem=True, version='4.4.4')
      +-- AnyNode(license='GPL', license_problem=True, name='package6', subtree_problem=False, version='6.6.6')

  print(RenderTree(tree, style=AsciiStyle()))
  """
    tree = AnyNode(name="Name", version="", license="MIT")
    # first level
    package1 = AnyNode(
        name="@company/package1", parent=tree, version="1.1.1", license="MIT"
    )
    package4 = AnyNode(name="package4", parent=tree, version="4.4.4", license="MIT")

    package2 = AnyNode(name="package2", parent=package1, version="2.2.2", license="MIT")
    AnyNode(name="package5", parent=package2, version="5.5.5", license="MIT")
    AnyNode(name="package7", parent=package2, version="7.7.7", license="MIT")

    package3 = AnyNode(name="package3", parent=package1, version="3.3.3", license="MIT")
    AnyNode(name="package7", parent=package3, version="7.7.6", license="GPL")

    AnyNode(name="package4", parent=package1, version="4.4.4", license="MIT")

    package5 = AnyNode(name="package5", parent=package1, version="5.5.5", license="MIT")
    AnyNode(name="package6", parent=package5, version="6.6.6", license="GPL")

    AnyNode(name="package6", parent=package4, version="6.6.6", license="GPL")
    return tree


class NpmRunnerTestCase(unittest.TestCase):
    def test_dependency_tree_is_flattened(self):
        tree = get_tree()
        dependencies = flatten_dependency_tree(tree)
        self.assertSetEqual(
            dependencies,
            {
                ("@company/package1", "1.1.1"),
                ("package2", "2.2.2"),
                ("package3", "3.3.3"),
                ("package4", "4.4.4"),
                ("package5", "5.5.5"),
                ("package6", "6.6.6"),
                ("package7", "7.7.6"),
                ("package7", "7.7.7"),
            },
        )

    def test_bad_licenses_identified(self):
        tree = get_tree()

        whitelist = ["MIT", "Apache-2.0"]
        _, unknown_licenses = annotate_dep_tree(tree, whitelist)
        self.assertSetEqual(set({"GPL"}), unknown_licenses)

    def test_annotate_dependency_tree(self):
        tree = get_tree()

        whitelist = ["MIT", "Apache-2.0"]
        annotated_tree, _ = annotate_dep_tree(tree, whitelist)

        expected_tree = AnyNode(
            name="Name",
            version="",
            license="MIT",
            license_problem=False,
            subtree_problem=True,
        )
        # first level
        package1 = AnyNode(
            name="@company/package1",
            parent=expected_tree,
            version="1.1.1",
            license="MIT",
            license_problem=False,
            subtree_problem=True,
        )
        package4 = AnyNode(
            name="package4",
            parent=expected_tree,
            version="4.4.4",
            license="MIT",
            license_problem=False,
            subtree_problem=True,
        )

        package2 = AnyNode(
            name="package2",
            parent=package1,
            version="2.2.2",
            license="MIT",
            license_problem=False,
            subtree_problem=False,
        )
        AnyNode(
            name="package5",
            parent=package2,
            version="5.5.5",
            license="MIT",
            license_problem=False,
            subtree_problem=False,
        )
        AnyNode(
            name="package7",
            parent=package2,
            version="7.7.7",
            license="MIT",
            license_problem=False,
            subtree_problem=False,
        )

        package3 = AnyNode(
            name="package3",
            parent=package1,
            version="3.3.3",
            license="MIT",
            license_problem=False,
            subtree_problem=True,
        )
        AnyNode(
            name="package7",
            parent=package3,
            version="7.7.6",
            license="GPL",
            license_problem=True,
            subtree_problem=False,
        )

        AnyNode(
            name="package4",
            parent=package1,
            version="4.4.4",
            license="MIT",
            license_problem=False,
            subtree_problem=False,
        )

        package5 = AnyNode(
            name="package5",
            parent=package1,
            version="5.5.5",
            license="MIT",
            license_problem=False,
            subtree_problem=True,
        )
        AnyNode(
            name="package6",
            parent=package5,
            version="6.6.6",
            license="GPL",
            license_problem=True,
            subtree_problem=False,
        )

        AnyNode(
            name="package6",
            parent=package4,
            version="6.6.6",
            license="GPL",
            license_problem=True,
            subtree_problem=False,
        )

        exporter = DictExporter()
        self.assertDictEqual(
            exporter.export(expected_tree), exporter.export(annotated_tree)
        )


if __name__ == "__main__":
    unittest.main()



import tempfile
from contextlib import nullcontext
from os import path
import xml.etree.ElementTree as ET
import subprocess

from anytree import AnyNode, PreOrderIter
from yaspin import yaspin
from typing import Dict
from pathlib import Path
from importlib import resources

from license_sh.runners import maven

DEPENDENCY_JAR = path.join(
    Path(__file__).parent,
    "..",
    "..",
    "..",
    "jar",
    "maven-dependency-plugin-3.1.1-Licensesh.jar",
)
GROUP_ID = "org.apache.maven.plugins"
ARTIFACT_ID = "maven-dependency-plugin"
VERSION = "3.1.1-Licensesh"
MULTI_LICENSE_JOIN = " AND "


def get_dependency_tree_xml(directory: str, debug=False):
    """Get maven dependency tree as xml
  
    Arguments:
      directory {str} -- path to maven project
  
    Returns:
      [xml] -- XML representation of maven dependency tree
    """

    with resources.path(maven, "pom.xml") as maven_path:
        subprocess.run(["mvn", "install", f"-f={maven_path}"], capture_output=not debug)
    with resources.path(
        maven, "maven-dependency-plugin-3.1.1-Licensesh.jar"
    ) as maven_path:
        subprocess.run(
            [
                "mvn",
                "install:install-file",
                f"-Dfile={maven_path}",
                f"-DgroupId={GROUP_ID}",
                f"-DartifactId={ARTIFACT_ID}",
                f"-Dversion={VERSION}",
                f"-Dpackaging=jar",
            ],
            capture_output=not debug,
        )

    with tempfile.NamedTemporaryFile() as tmpfile:
        subprocess.run(
            [
                "mvn",
                f"{GROUP_ID}:{ARTIFACT_ID}:{VERSION}:tree",
                f"-DoutputType=xml",
                f"-DoutputFile={tmpfile.name}",
                "-f",
                directory,
            ],
            capture_output=not debug,
        )
        return ET.parse(tmpfile).getroot()

    return None


def get_license_xml_file(directory: str, debug: bool) -> ET.ElementTree:
    """Get maven xml licenses

    Arguments:
      directory {str} -- path to maven project

    Returns:
      [xml] -- XML representation of maven licenses xml
    """
    with tempfile.TemporaryDirectory() as dirpath:
        fname = path.join(dirpath, "licenses.tmp")
        subprocess.run(
            [
                "mvn",
                "license:download-licenses",
                f"-DlicensesOutputDirectory={dirpath}",
                "-DskipDownloadLicenses=true",
                f"-DlicensesOutputFile={fname}",
                "-f",
                directory,
            ],
            capture_output=not debug,
        )
        return ET.parse(fname).getroot()
    return None


def get_project_name(pom_xml) -> str:
    """Get project name(artifactID)
  
  Arguments:
      pom_xml {xml} -- xml representation of pom.xml
  
  Returns:
      str -- project name parsed from pom.xml
  """
    for child in pom_xml:
        if "artifactId" in child.tag:
            return child.text


def get_project_pom_xml(directory: str):
    """Get xml representation of pom.xml
  
  Arguments:
      directory {str} -- path to maven project
  
  Returns:
      [xml] -- xml representation of pom.xml
  """
    return ET.parse(path.join(directory, "pom.xml")).getroot()


def parse_licenses_xml(xml) -> Dict[str, str]:
    """Parse xml representation of maven licenses xml

  Example:
    {
      "package@1.2.3": "MIT",
      "utilsPackage@1.2.3": "GPL",
      "renderPackage@1.1.1": "APACHE"
    }
  
  
  Arguments:
      xml -- xml representation of maven licenses xml
  
  Returns:
      Dict[str, str] -- Dict with NAME@VERSION as key and license as value
  """
    license_map = {}
    for dependency in xml.find("dependencies"):
        name = "@".join(
            [dependency.find("artifactId").text, dependency.find("version").text]
        )
        licenses = dependency.find("licenses")
        license_name = None
        for license in licenses:
            if license_name is None:
                license_name = license.find("name").text
            else:
                license_name = "({})".format(
                    " AND ".join([license_name, license.find("name").text])
                )
        license_map[name] = license_name

    return license_map


def parse_dependency_xml(xml, parent: AnyNode = None) -> AnyNode:
    """Parse xml representation of maven dependency tree
  
  Arguments:
      xml -- xml representation of maven dependency tree
  
  Keyword Arguments:
      parent {AnyNode} -- Parent node, used in recursive parsing (default: {None})
  
  Returns:
      [AnyNode] -- Parsed xml dependency tree
  """
    if xml is None:
        return None
    root = AnyNode(name=xml.tag, version=xml.get("version"), parent=parent)
    for dependency in xml:
        parse_dependency_xml(dependency, root)
    return root


class MavenRunner:
    """
  This class checks for dependencies in maven projects and fetches license info
  for each of the packages (including transitive dependencies)
  """

    def __init__(self, directory: str, silent: bool, debug: bool):
        self.directory = directory
        self.silent = silent
        self.debug = debug

    def check(self):
        project_name = get_project_name(get_project_pom_xml(self.directory))

        if not self.silent:
            print("===========")
            print(
                f"Initiated License.sh check for Maven project {project_name} located at {self.directory}"
            )
            print("===========")

        with (
            yaspin(text="Getting dependency tree... (First run might take a while)")
            if not self.silent
            else nullcontext()
        ) as sp:
            xml_tree = get_dependency_tree_xml(self.directory, self.debug)

        with (
            yaspin(text="Analysing dependencies ...")
            if not self.silent
            else nullcontext()
        ) as sp:
            dep_tree = parse_dependency_xml(xml_tree)
            license_map = parse_licenses_xml(
                get_license_xml_file(self.directory, self.debug)
            )

        for node in PreOrderIter(dep_tree):
            node.license = license_map.get(f"{node.name}@{node.version}", "")

        return dep_tree, license_map



from anytree import Node, RenderTree, AnyNode, ContStyle
from license_sh.helpers import GREEN, RESET, RED


class ConsoleReporter:
    @staticmethod
    def output(dependency_tree):
        for pre, fill, node in RenderTree(dependency_tree, style=ContStyle()):
            if node.is_root:
                print(" ")
            else:
                color = RED if node.license_problem else GREEN
                print(
                    f"{pre}{node.name} - {node.version} - {color}{node.license}{RESET}"
                )



import json
import subprocess
from os import path
from contextlib import nullcontext

from anytree import AnyNode, PreOrderIter

from yaspin import yaspin

from license_sh.runners.runners_shared import fetch_npm_licenses


def flatten_package_lock_dependencies(package_lock_dependencies):
    """
  Flattens package.lock dependencies
  :param package_lock_dependencies:
  :return: tuple(flat_array, dep_mapping)
     WHERE
     array flat_array is a flat array of all dependencies
     dict dep_mapping is a tree representing nested dependencies
  """
    # get the root dependencies
    root_deps = [
        (name, dep.get("version")) for name, dep in package_lock_dependencies.items()
    ]

    for dep in package_lock_dependencies.values():
        version, requires, dependencies = [
            dep.get(k, None) for k in ("version", "requires", "dependencies")
        ]
        if dependencies:
            deps = flatten_package_lock_dependencies(dependencies)
            root_deps += deps

    return set(root_deps)


def add_nested_dependencies(dependency, package_lock_tree, parent):
    for name, version_request in dependency.get("requires", {}).items():
        node = AnyNode(
            name=name,
            version_request=version_request,
            parent=parent,
            dependencies=dependency.get("dependencies"),
        )

        dep = None
        p = node
        names = []
        while p.parent:
            if dep is None and p.dependencies and p.dependencies.get(name):
                dep = p.dependencies.get(name)
            p = p.parent
            names.append(p.name)

        # check the root
        if dep is None and p.dependencies and p.dependencies.get(name):
            dep = p.dependencies.get(name)

        node.version = dep.get("version")

        names = names[:-1]  # let's forget about top level

        # if I'm not already in a tree
        if name not in names:
            if dep:
                add_nested_dependencies(dep, package_lock_tree, node)


def get_dependency_tree(package_json, package_lock_tree):
    root = AnyNode(
        name=package_json.get("name", "package.json"),
        dependencies=package_lock_tree,
        version=package_json.get("version"),
    )

    # load root dependencies from package.json
    for dep_name in package_json.get("dependencies", {}).keys():
        dependency = package_lock_tree[dep_name]
        version = dependency.get("version")
        parent = AnyNode(
            name=dep_name,
            version=version,
            parent=root,
            dependencies=dependency.get("dependencies"),
        )
        add_nested_dependencies(dependency, package_lock_tree, parent=parent)

    return root


def fetch_license(dep):
    dependency, version = dep
    result = subprocess.run(
        ["npm", "info", f"{dependency}@{version}", "--json"], stdout=subprocess.PIPE
    )
    info = json.loads(result.stdout)
    return 3


class NpmRunner:
    """
  This class checks for dependencies in NPM projects and fetches license info
  for each of the packages (including transitive dependencies)
  """

    def __init__(self, directory: str, silent: bool, debug: bool):
        self.directory = directory
        self.silent = silent
        self.package_json_path = path.join(directory, "package.json")
        self.package_lock_path = path.join(directory, "package-lock.json")
        self.debug = debug

    def check(self):
        with open(self.package_json_path) as package_json_file:
            package_json = json.load(package_json_file)
            project_name = package_json.get("name", "package.json")

        with open(self.package_lock_path) as package_lock_file:
            package_lock = json.load(package_lock_file)
            all_dependencies = package_lock["dependencies"]
        if not self.silent:
            print("===========")
            print(
                f"Initiated License.sh check for NPM project {project_name} located at {self.directory}"
            )
            print("===========")

        with (
            yaspin(text="Analysing dependencies ...")
            if not self.silent
            else nullcontext()
        ) as sp:
            dep_tree = get_dependency_tree(package_json, all_dependencies)
            flat_dependencies = flatten_package_lock_dependencies(all_dependencies)

        with (
            yaspin(text="Fetching license info from npm ...")
            if not self.silent
            else nullcontext()
        ) as sp:
            license_map = fetch_npm_licenses(flat_dependencies)

        for node in PreOrderIter(dep_tree):
            delattr(node, "dependencies")
            hasattr(node, "version_request") and delattr(node, "version_request")
            node.license = license_map.get(f"{node.name}@{node.version}", None)

        return dep_tree, license_map



import asyncio
import json
import os
import subprocess
from json import JSONDecodeError
from contextlib import nullcontext

import aiohttp
from anytree import AnyNode, PreOrderIter
from yaspin import yaspin

from license_sh.helpers import flatten_dependency_tree


def add_nested_dependencies(dep, parent):
    name = dep["package_name"]
    version = dep["installed_version"]
    depdendencies = dep["dependencies"]

    node = AnyNode(name=name, version=version, parent=parent)
    for dep in depdendencies:
        add_nested_dependencies(dep, node)


PYPI_HOST = "https://pypi.org/pypi"


class PythonRunner:
    def __init__(self, directory: str, silent: bool, debug: bool):
        self.directory = directory
        self.silent = silent
        self.pipfile_path: str = os.path.join(self.directory, "Pipfile")
        self.pipfile_lock_path: str = os.path.join(self.directory, "Pipfile.lock")
        self.debug = debug

    @staticmethod
    def fetch_licenses(all_dependencies):
        license_map = {}

        urls = [
            f"{PYPI_HOST}/{dependency}/{version}/json"
            for dependency, version in all_dependencies
        ]

        async def fetch(session, url):
            async with session.get(url) as resp:
                return await resp.text()
                # Catch HTTP errors/exceptions here

        async def fetch_concurrent(urls):
            loop = asyncio.get_event_loop()
            async with aiohttp.ClientSession() as session:
                tasks = []
                for u in urls:
                    tasks.append(loop.create_task(fetch(session, u)))

                for result in asyncio.as_completed(tasks):
                    try:
                        page = json.loads(await result)
                        info = page.get("info", {})
                        license_map[
                            f"{info.get('name')}@{info.get('version')}"
                        ] = info.get("license", "Unknown")
                    except JSONDecodeError:
                        # TODO - investiage why does such a thing happen
                        pass

        asyncio.run(fetch_concurrent(urls))

        return license_map

    def check(self):
        if not self.silent:
            print("===========")
            print(
                f"Initiated License.sh check for pipenv project located at {self.directory}"
            )
            print("===========")

        with (
            yaspin(text="Analysing dependencies ...")
            if not self.silent
            else nullcontext()
        ) as sp:
            result = subprocess.run(
                ["pipdeptree", "--json-tree", "--local-only"],
                capture_output=not self.debug,
            )
            dep_tree = json.loads(result.stdout)

            root = AnyNode(name="root", version="")

            for dep in dep_tree:
                add_nested_dependencies(dep, root)

            all_dependencies = flatten_dependency_tree(root)

        with (
            yaspin(text="Fetching license info from pypi ...")
            if not self.silent
            else nullcontext()
        ) as sp:
            license_map = PythonRunner.fetch_licenses(all_dependencies)

        for node in PreOrderIter(root):
            node.license = license_map.get(f"{node.name}@{node.version}", None)

        return root, license_map



import json
import subprocess
from os import path
from anytree import AnyNode, PreOrderIter
from typing import Dict, List
from yaspin import yaspin
from contextlib import nullcontext

from importlib import resources

from license_sh.runners.runners_shared import fetch_npm_licenses
from license_sh.runners.yarn import js


def get_yarn_list_json(pathToYarn: str) -> Dict:
    """Get result of 'yarn list --json --silent --no-progress' as json

    Arguments:
        path {str} -- Path to yarn project

    Returns:
        Dict -- Parsed json of command result
    """
    return json.loads(
        (
            subprocess.run(
                [
                    "yarn",
                    "list",
                    "--json",
                    "--silent",
                    "--no-progress",
                    "--cwd",
                    pathToYarn,
                ],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
        ).stdout
    )


def get_yarn_lock_json(pathToYarn: str) -> Dict:
    """Get yarn lock file as json

    Arguments:
      path {str} -- Path to yarn project

    Returns:
      Dict -- Parsed yarn lock json
    """

    with resources.path(js, ".") as js_path:
        # install script dependency
        subprocess.run(
            ["yarn", "install", "--frozen-lockfile", "--cwd", js_path],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        # run yarn lock parser
        res = json.loads(
            subprocess.run(
                [
                    "node",
                    path.join(js_path, "parseYarnLock.js"),
                    path.join(pathToYarn, "yarn.lock"),
                ],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            ).stdout
        )

        return res


def parse_yarn_lock(json_element: Dict) -> Dict[str, str]:
    """Parse yarn lock json into dict where
    keys are package name + required version and value is locked version
    Example:
      {
        "package@1.2.3": "1.2.3",
        "utilsPackage@^1.2.3": "1.5.0",
        "renderPackage@~1.1.1": "1.1.1"
      }

    Arguments:
        json_element {Dict} -- Root element of json yarn lock

    Returns:
        Dict[str, str] -- Parsed yarn lock into package dict
    """
    return {
        key: dependency.get("version", None)
        for key, dependency in json_element.get("object", {}).items()
    }


def get_name(name: str) -> str:
    """Get package name from package name + required version

    Example:

    package@1.2.3 -> package
    @bubolia/package@2.2.2 -> @bubolia/package

    Arguments:
        name {str} -- Package name to parse name from

    Returns:
        str -- Name of the package without required version
    """
    *name_arr_with_optional_at_sign, version = name.split("@")
    return "@".join(name_arr_with_optional_at_sign)


def get_flat_tree(dependencies: List, package_map: Dict[str, str]) -> Dict:
    """Parse yarn list json dependencies and add them locked version

  Example:
  {
    "package@1.2.3": {
      "name": "package",
      "version": "1.2.3",
      dependencies: {
        "utils@1.2.3": {
          "name": "utils",
          "version": "1.2.3",
          dependencies: {}
        }
      }
    },
    "helper@3.2.1": {
      "name": "helper",
      "version": "3.2.1",
      dependencies: {}
    }
  }

  Arguments:
      dependencies {List} -- List of Dicts that represent yarn list.
      package_map {Dict[str, str]} -- Dict to resolve locked versions of packages

  Returns:
      Dict -- Dict representation of yarn flat tree.
  """
    flat_tree = {}
    for dependency in dependencies:
        dep_full_name = dependency.get("name")
        flat_tree[dependency.get("name")] = {
            "name": get_name(dep_full_name),
            "version": package_map.get(dep_full_name),
            "dependencies": get_flat_tree(dependency.get("children", []), package_map),
        }
    return flat_tree


def get_node_from_dependency(dependency: Dict, parent: AnyNode) -> AnyNode:
    """Parse dependency dict into node

  Dependency example:
   {
      "name": "utils",
      "version": "1.2.3",
      "dependencies": {}
    }

  Arguments:
      dependency {Dict} -- Dict representation of dependency
      parent {AnyNode} -- Parent node

  Returns:
      AnyNode -- Parsed node or None if not valid dependency
  """
    dep_version = dependency.get("version")
    dep_name = dependency.get("name")
    if not dep_version or not dep_name:
        return None
    return AnyNode(
        name=dependency.get("name"),
        version=dependency.get("version"),
        parent=parent,
        dependencies=dependency.get("dependencies", {}),
    )


def find_full_dependency(dependencies: Dict, name: str) -> Dict:
    """Find full dependency in dependencies dict

  Full dependency mean dependency that has childs specified

  Arguments:
      dependencies {Dict} -- Dict representation of dependencies, key as name, value as dependency
      name {str} -- Name of node to find

  Returns:
      Dict -- Found dependency or None
  """
    if not dependencies:
        return None
    dependency = dependencies.get(name)
    if not dependency:
        return None
    result = dependency.get("dependencies")
    if result:
        return dependency
    return None


def add_nested_dependencies(dependency: Dict, parent: AnyNode) -> None:
    """Recurcivelly resolve nested dependencies

  Dependency example:
   {
      "name": "utils",
      "version": "1.2.3",
      "dependencies": {}
    }

  Arguments:
      dependency {Dict} -- Dict representation of dependency
      parent {AnyNode} -- Parent node
  """
    for dependency in dependency.get("dependencies", {}).values():
        node = get_node_from_dependency(dependency, parent)
        if not node:
            continue
        dep_name = f"{node.name}@{node.version}"
        dep = None
        checkNode = node.parent
        names = []
        while checkNode.parent:
            full_dep = find_full_dependency(checkNode.dependencies, dep_name)
            if not dep and full_dep:
                node.dependencies = full_dep.get(
                    "dependencies"
                )  # Update own dependencies with correct dependencies
                dep = full_dep

            checkNode = checkNode.parent  # continue for parent
            names.append(checkNode.name)  # save name to prevent cyclic dependency loop

        # Check the root
        full_dep = find_full_dependency(checkNode.dependencies, dep_name)
        if not dep and full_dep:
            node.dependencies = node.dependencies = full_dep.get("dependencies")
            dep = full_dep

        names = names[:-1]  # let's forget about top level
        # If I'm not already in a tree
        if node.name not in names and dep:
            add_nested_dependencies(dep, node)


def get_dependency_tree(
    flat_tree: Dict, package_json: Dict, package_map: Dict
) -> AnyNode:
    """Get dependency tree.

  Arguments:
      flat_tree {Dict} -- Yarn flat tree
      package_json {Dict} -- package.json of yarn project
      package_map {Dict} -- package_map with locked versions

  Returns:
      AnyNode -- Dependency tree
  """
    # Create root node with project info
    root = AnyNode(
        name=package_json.get("name", "package.json"),
        dependencies=flat_tree,
        version=package_json.get("version"),
    )
    # Load root dependencies from package.json
    for dep_name, dep_version in package_json.get("dependencies", {}).items():
        resolved_version = package_map.get(
            f"{dep_name}@{dep_version}"
        )  # Resolve locked version of the package
        dependency = flat_tree.get(
            f"{dep_name}@{resolved_version}"
        )  # Use package full name to get it dependencies
        # Create first level of nodes based on package.json dependencies
        parent = AnyNode(
            name=dep_name,
            version=dependency.get("version"),
            parent=root,
            dependencies=dependency.get("dependencies"),
        )
        add_nested_dependencies(
            dependency, parent
        )  # Add nested dependencies of first level nodes

    # Delete helper dependencies field
    for node in PreOrderIter(root):
        delattr(node, "dependencies")

    return root


class YarnRunner:
    """
    This class checks for dependencies in Yarn projects and fetches license info
    for each of the packages (including transitive dependencies)
    """

    def __init__(self, directory: str, silent: bool, debug: bool):
        self.directory = directory
        self.silent = silent
        self.package_json_path = path.join(directory, "package.json")
        self.yarn_lock_path = path.join(directory, "yarn.lock")
        self.debug = debug

    def check(self):
        with open(self.package_json_path) as package_json_file:
            package_json = json.load(package_json_file)
            project_name = package_json.get("name", "project_name")

        if not self.silent:
            print("===========")
            print(
                f"Initiated License.sh check for YARN project {project_name} located at {self.directory}"
            )
            print("===========")

        with (
            yaspin(text="Analysing dependencies ...")
            if not self.silent
            else nullcontext()
        ) as sp:
            package_map = parse_yarn_lock(get_yarn_lock_json(self.directory))
            flat_tree = get_flat_tree(
                get_yarn_list_json(self.directory).get("data", {}).get("trees", []),
                package_map,
            )
            dep_tree = get_dependency_tree(flat_tree, package_json, package_map)

            flat_dependencies = [s.split("@") for s in flat_tree.keys()]
            license_map = fetch_npm_licenses(flat_dependencies)

            for node in PreOrderIter(dep_tree):
                node.license = license_map.get(f"{node.name}@{node.version}", None)

        return dep_tree, license_map



from anytree.exporter import JsonExporter


class JSONConsoleReporter:
    @staticmethod
    def output(dependency_tree):
        exporter = JsonExporter(indent=2)
        print(exporter.export(dependency_tree))
        return 0



#!/usr/bin/env python
import setuptools
from version import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="license-sh",
    version=__version__,
    author="Jan Vorcak",
    author_email="vorcak@webscope.io",
    description="Verify software licenses of your open source software",
    scripts=["license-sh"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/webscopeio/license.sh",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    package_data={
        "license_sh": [
            "runners/yarn/js/package.json",
            "runners/yarn/js/yarn.lock",
            "runners/yarn/js/parseYarnLock.js",
            "runners/maven/maven-dependency-plugin-3.1.1-Licensesh.jar",
            "runners/maven/pom.xml",
        ]
    },
    license="MIT",
    install_requires=[
        "anytree",
        "yaspin",
        "aiohttp",
        "PyInquirer",
        "urllib3",
        "docopt",
        "license-expression",
        "pipdeptree",
    ],
)



import json
import os
from typing import Tuple

from PyInquirer import Separator, prompt

from license_sh.config import write_config
from license_sh.project_identifier import get_project_types


def config_cmd(path, config):
    projects = config.get("projects", [])
    whitelist = config.get("whitelist", [])

    choices = []
    for dir_name, subdir_list, file_list in os.walk(path, followlinks=False):
        project_types = get_project_types(dir_name)
        if project_types:
            choices.append(Separator(f"== Path: {dir_name} =="))
            for project_type in project_types:
                choices.append(
                    {
                        "name": f"{project_type.value} project",
                        "value": {"directory": dir_name, "type": project_type.value},
                    }
                )

    default_licenses = [
        "MIT",
        "Apache-2.0",
        "BSD",
        "GPL-2.0",
        "GPL-3.0",
        "LGPL-3.0",
        "VSPL",
        "MPL-2.0",
        "FreeBSD",
        "Zlib",
        "AFL",
        "X11",
        "JSON",
    ]

    questions = [
        {
            "type": "confirm",
            "message": "Do you want to modify project paths to be watched by this tool?",
            "name": "change_projects",
            "default": False,
            "when": lambda x: len(projects) > 0,
        },
        {
            "type": "checkbox",
            "qmark": "📦",
            "message": "Select projects to watch",
            "name": "projects",
            "choices": choices,
            "when": lambda x: len(projects) == 0 or x.get("change_projects"),
        },
        {
            "type": "checkbox",
            "qmark": "📋",
            "message": "Select licenses which are OK to be used for your project",
            "name": "whitelist",
            "choices": [
                {"name": license, "checked": license in whitelist}
                for license in default_licenses
            ],
        },
    ]
    answers = prompt(questions)

    if "projects" in answers:
        config["projects"] = answers["projects"]
    if "whitelist" in answers:
        config["whitelist"] = answers["whitelist"]

    if write_config(path, config):
        print("Successfully generated .license-sh file")



from typing import Tuple, Set

from anytree import PreOrderIter, LevelOrderIter, AnyNode
from license_expression import Licensing

licensing = Licensing()

RED = "\033[1;31m"
BLUE = "\033[1;34m"
CYAN = "\033[1;36m"
GREEN = "\033[0;32m"
RESET = "\033[0;0m"
BOLD = "\033[;1m"
REVERSE = "\033[;7m"


def flatten_dependency_tree(tree):
    # remove the root node
    return set(
        [(node.name, node.version) for node in PreOrderIter(tree) if tree is not node]
    )


def is_license_ok(license_text, whitelist):
    """
  Identifies whether license is compliant with the whitelist
  :param license_text: string or licensing object
  :param whitelist: list of strings to compare with
  :return:
  True - if license is compliant.
  False - if it's not compliant.
  None - if there is a parsing error.
  """
    try:
        license = licensing.parse(license_text)
    except:
        return license_text in whitelist

    if license is None:
        return None

    if license.isliteral:
        return license.render() in whitelist

    operator = license.operator.strip()

    fn = {"OR": any, "AND": all}[operator]

    return fn(map(lambda x: is_license_ok(x, whitelist), license.args))


def annotate_dep_tree(tree, whitelist: [str]) -> Tuple[AnyNode, Set[str]]:
    """
  An idea of this function is to go through elements from the bottom -> up and
  mark subtree_problem if any of the children has a license_problem or a subtree_problem
  :param tree:
  :param whitelist:
  :return: list of licenses not found in a whitelist
  """

    licenses_not_found = set()
    for node in PreOrderIter(tree):
        node.license_problem = not is_license_ok(node.license, whitelist)
        if node.license_problem and node.license:
            licenses_not_found.add(node.license)

    for node in list(LevelOrderIter(tree))[::-1]:
        node.subtree_problem = (
            False
            if node.is_leaf
            else any(
                map(lambda x: x.subtree_problem or x.license_problem, node.children)
            )
        )

    return tree, licenses_not_found


def filter_dep_tree(tree: AnyNode) -> AnyNode:
    """Filter dependency tree.
    
    Leave only nodes with license problem of itself or children 
    
    Arguments:
        tree {AnyNode} -- Tree to filter
    
    Returns:
        AnyNode -- Filtered tree
    """
    for node in LevelOrderIter(tree):
        node.children = filter(
            lambda subnode: subnode.subtree_problem or subnode.license_problem,
            node.children,
        )

    return tree
